/*
 * Generated by gdbus-codegen 2.43.1. DO NOT EDIT.
 *
 * The license of this code is the same as for the source it was derived from.
 */

#ifndef __DBUS_KEY_GRABBER_H__
#define __DBUS_KEY_GRABBER_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Shell */

#define DBUS_TYPE_KEY_GRABBER (dbus_key_grabber_get_type ())
#define DBUS_KEY_GRABBER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_KEY_GRABBER, DBusKeyGrabber))
#define DBUS_IS_KEY_GRABBER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_KEY_GRABBER))
#define DBUS_KEY_GRABBER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), DBUS_TYPE_KEY_GRABBER, DBusKeyGrabberIface))

struct _DBusKeyGrabber;
typedef struct _DBusKeyGrabber DBusKeyGrabber;
typedef struct _DBusKeyGrabberIface DBusKeyGrabberIface;

struct _DBusKeyGrabberIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_grab_accelerator) (
    DBusKeyGrabber *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_accelerator,
    guint arg_flags);

  gboolean (*handle_grab_accelerators) (
    DBusKeyGrabber *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_accelerators);

  gboolean (*handle_ungrab_accelerator) (
    DBusKeyGrabber *object,
    GDBusMethodInvocation *invocation,
    guint arg_action);

  void (*accelerator_activated) (
    DBusKeyGrabber *object,
    guint arg_action,
    guint arg_device,
    guint arg_timestamp);

};

GType dbus_key_grabber_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *dbus_key_grabber_interface_info (void);
guint dbus_key_grabber_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void dbus_key_grabber_complete_grab_accelerator (
    DBusKeyGrabber *object,
    GDBusMethodInvocation *invocation,
    guint action);

void dbus_key_grabber_complete_grab_accelerators (
    DBusKeyGrabber *object,
    GDBusMethodInvocation *invocation,
    GVariant *actions);

void dbus_key_grabber_complete_ungrab_accelerator (
    DBusKeyGrabber *object,
    GDBusMethodInvocation *invocation,
    gboolean success);



/* D-Bus signal emissions functions: */
void dbus_key_grabber_emit_accelerator_activated (
    DBusKeyGrabber *object,
    guint arg_action,
    guint arg_device,
    guint arg_timestamp);



/* D-Bus method calls: */
void dbus_key_grabber_call_grab_accelerator (
    DBusKeyGrabber *proxy,
    const gchar *arg_accelerator,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_key_grabber_call_grab_accelerator_finish (
    DBusKeyGrabber *proxy,
    guint *out_action,
    GAsyncResult *res,
    GError **error);

gboolean dbus_key_grabber_call_grab_accelerator_sync (
    DBusKeyGrabber *proxy,
    const gchar *arg_accelerator,
    guint arg_flags,
    guint *out_action,
    GCancellable *cancellable,
    GError **error);

void dbus_key_grabber_call_grab_accelerators (
    DBusKeyGrabber *proxy,
    GVariant *arg_accelerators,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_key_grabber_call_grab_accelerators_finish (
    DBusKeyGrabber *proxy,
    GVariant **out_actions,
    GAsyncResult *res,
    GError **error);

gboolean dbus_key_grabber_call_grab_accelerators_sync (
    DBusKeyGrabber *proxy,
    GVariant *arg_accelerators,
    GVariant **out_actions,
    GCancellable *cancellable,
    GError **error);

void dbus_key_grabber_call_ungrab_accelerator (
    DBusKeyGrabber *proxy,
    guint arg_action,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_key_grabber_call_ungrab_accelerator_finish (
    DBusKeyGrabber *proxy,
    gboolean *out_success,
    GAsyncResult *res,
    GError **error);

gboolean dbus_key_grabber_call_ungrab_accelerator_sync (
    DBusKeyGrabber *proxy,
    guint arg_action,
    gboolean *out_success,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define DBUS_TYPE_KEY_GRABBER_PROXY (dbus_key_grabber_proxy_get_type ())
#define DBUS_KEY_GRABBER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_KEY_GRABBER_PROXY, DBusKeyGrabberProxy))
#define DBUS_KEY_GRABBER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_KEY_GRABBER_PROXY, DBusKeyGrabberProxyClass))
#define DBUS_KEY_GRABBER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_KEY_GRABBER_PROXY, DBusKeyGrabberProxyClass))
#define DBUS_IS_KEY_GRABBER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_KEY_GRABBER_PROXY))
#define DBUS_IS_KEY_GRABBER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_KEY_GRABBER_PROXY))

typedef struct _DBusKeyGrabberProxy DBusKeyGrabberProxy;
typedef struct _DBusKeyGrabberProxyClass DBusKeyGrabberProxyClass;
typedef struct _DBusKeyGrabberProxyPrivate DBusKeyGrabberProxyPrivate;

struct _DBusKeyGrabberProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  DBusKeyGrabberProxyPrivate *priv;
};

struct _DBusKeyGrabberProxyClass
{
  GDBusProxyClass parent_class;
};

GType dbus_key_grabber_proxy_get_type (void) G_GNUC_CONST;

void dbus_key_grabber_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
DBusKeyGrabber *dbus_key_grabber_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
DBusKeyGrabber *dbus_key_grabber_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void dbus_key_grabber_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
DBusKeyGrabber *dbus_key_grabber_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
DBusKeyGrabber *dbus_key_grabber_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define DBUS_TYPE_KEY_GRABBER_SKELETON (dbus_key_grabber_skeleton_get_type ())
#define DBUS_KEY_GRABBER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_KEY_GRABBER_SKELETON, DBusKeyGrabberSkeleton))
#define DBUS_KEY_GRABBER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_KEY_GRABBER_SKELETON, DBusKeyGrabberSkeletonClass))
#define DBUS_KEY_GRABBER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_KEY_GRABBER_SKELETON, DBusKeyGrabberSkeletonClass))
#define DBUS_IS_KEY_GRABBER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_KEY_GRABBER_SKELETON))
#define DBUS_IS_KEY_GRABBER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_KEY_GRABBER_SKELETON))

typedef struct _DBusKeyGrabberSkeleton DBusKeyGrabberSkeleton;
typedef struct _DBusKeyGrabberSkeletonClass DBusKeyGrabberSkeletonClass;
typedef struct _DBusKeyGrabberSkeletonPrivate DBusKeyGrabberSkeletonPrivate;

struct _DBusKeyGrabberSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  DBusKeyGrabberSkeletonPrivate *priv;
};

struct _DBusKeyGrabberSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType dbus_key_grabber_skeleton_get_type (void) G_GNUC_CONST;

DBusKeyGrabber *dbus_key_grabber_skeleton_new (void);


/* ---- */

#define DBUS_TYPE_OBJECT (dbus_object_get_type ())
#define DBUS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_OBJECT, DBusObject))
#define DBUS_IS_OBJECT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_OBJECT))
#define DBUS_OBJECT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), DBUS_TYPE_OBJECT, DBusObject))

struct _DBusObject;
typedef struct _DBusObject DBusObject;
typedef struct _DBusObjectIface DBusObjectIface;

struct _DBusObjectIface
{
  GTypeInterface parent_iface;
};

GType dbus_object_get_type (void) G_GNUC_CONST;

DBusKeyGrabber *dbus_object_get_key_grabber (DBusObject *object);
DBusKeyGrabber *dbus_object_peek_key_grabber (DBusObject *object);

#define DBUS_TYPE_OBJECT_PROXY (dbus_object_proxy_get_type ())
#define DBUS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_OBJECT_PROXY, DBusObjectProxy))
#define DBUS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_OBJECT_PROXY, DBusObjectProxyClass))
#define DBUS_OBJECT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_OBJECT_PROXY, DBusObjectProxyClass))
#define DBUS_IS_OBJECT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_OBJECT_PROXY))
#define DBUS_IS_OBJECT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_OBJECT_PROXY))

typedef struct _DBusObjectProxy DBusObjectProxy;
typedef struct _DBusObjectProxyClass DBusObjectProxyClass;
typedef struct _DBusObjectProxyPrivate DBusObjectProxyPrivate;

struct _DBusObjectProxy
{
  /*< private >*/
  GDBusObjectProxy parent_instance;
  DBusObjectProxyPrivate *priv;
};

struct _DBusObjectProxyClass
{
  GDBusObjectProxyClass parent_class;
};

GType dbus_object_proxy_get_type (void) G_GNUC_CONST;
DBusObjectProxy *dbus_object_proxy_new (GDBusConnection *connection, const gchar *object_path);

#define DBUS_TYPE_OBJECT_SKELETON (dbus_object_skeleton_get_type ())
#define DBUS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_OBJECT_SKELETON, DBusObjectSkeleton))
#define DBUS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_OBJECT_SKELETON, DBusObjectSkeletonClass))
#define DBUS_OBJECT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_OBJECT_SKELETON, DBusObjectSkeletonClass))
#define DBUS_IS_OBJECT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_OBJECT_SKELETON))
#define DBUS_IS_OBJECT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_OBJECT_SKELETON))

typedef struct _DBusObjectSkeleton DBusObjectSkeleton;
typedef struct _DBusObjectSkeletonClass DBusObjectSkeletonClass;
typedef struct _DBusObjectSkeletonPrivate DBusObjectSkeletonPrivate;

struct _DBusObjectSkeleton
{
  /*< private >*/
  GDBusObjectSkeleton parent_instance;
  DBusObjectSkeletonPrivate *priv;
};

struct _DBusObjectSkeletonClass
{
  GDBusObjectSkeletonClass parent_class;
};

GType dbus_object_skeleton_get_type (void) G_GNUC_CONST;
DBusObjectSkeleton *dbus_object_skeleton_new (const gchar *object_path);
void dbus_object_skeleton_set_key_grabber (DBusObjectSkeleton *object, DBusKeyGrabber *interface_);

/* ---- */

#define DBUS_TYPE_OBJECT_MANAGER_CLIENT (dbus_object_manager_client_get_type ())
#define DBUS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_OBJECT_MANAGER_CLIENT, DBusObjectManagerClient))
#define DBUS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_OBJECT_MANAGER_CLIENT, DBusObjectManagerClientClass))
#define DBUS_OBJECT_MANAGER_CLIENT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_OBJECT_MANAGER_CLIENT, DBusObjectManagerClientClass))
#define DBUS_IS_OBJECT_MANAGER_CLIENT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_OBJECT_MANAGER_CLIENT))
#define DBUS_IS_OBJECT_MANAGER_CLIENT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_OBJECT_MANAGER_CLIENT))

typedef struct _DBusObjectManagerClient DBusObjectManagerClient;
typedef struct _DBusObjectManagerClientClass DBusObjectManagerClientClass;
typedef struct _DBusObjectManagerClientPrivate DBusObjectManagerClientPrivate;

struct _DBusObjectManagerClient
{
  /*< private >*/
  GDBusObjectManagerClient parent_instance;
  DBusObjectManagerClientPrivate *priv;
};

struct _DBusObjectManagerClientClass
{
  GDBusObjectManagerClientClass parent_class;
};

GType dbus_object_manager_client_get_type (void) G_GNUC_CONST;

GType dbus_object_manager_client_get_proxy_type (GDBusObjectManagerClient *manager, const gchar *object_path, const gchar *interface_name, gpointer user_data);

void dbus_object_manager_client_new (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *dbus_object_manager_client_new_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *dbus_object_manager_client_new_sync (
    GDBusConnection        *connection,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);

void dbus_object_manager_client_new_for_bus (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GAsyncReadyCallback     callback,
    gpointer                user_data);
GDBusObjectManager *dbus_object_manager_client_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
GDBusObjectManager *dbus_object_manager_client_new_for_bus_sync (
    GBusType                bus_type,
    GDBusObjectManagerClientFlags  flags,
    const gchar            *name,
    const gchar            *object_path,
    GCancellable           *cancellable,
    GError                **error);


G_END_DECLS

#endif /* __DBUS_KEY_GRABBER_H__ */
